.p02


.include "nes.h"
.import clearMap, clearLineImg, copyLineImg, lineImgBuf, putTile, vwfPuts
.export chrData

.segment "INESHDR"

  .byt "NES", 26
  .byt 2  ; number of 16 KB program segments
  .byt 0  ; number of 8 KB chr segments
  .byt $20  ; mapper, mirroring, etc
  .byt $20  ; extended mapper info
  .byt 0,0,0,0,0,0,0,0  ; f you DiskDude

.segment "ZEROPAGE"
vblanked:  .res 1
curLine: .res 1

.segment "CODE"

nmihandler:
  inc vblanked
irqhandler:
  rti

main:

;;; Init CPU
  sei
  cld
  ldx #$ff
  txs
  inx
  stx PPUCTRL
  stx PPUMASK

;;; Init PPU
:
  bit $2002
  bpl :-
:
  bit $2002
  bpl :-

;;; Load initial palette
  lda #$3f
  sta PPUADDR
  stx PPUADDR
@palclrloop:
  lda pal,x
  sta PPUDATA
  inx
  lda pal,x
  sta PPUDATA
  inx
  cpx #32
  bne @palclrloop

.if 0
;;; Load initial chr
  jsr clearLineImg
  ldy #0
  ldy #0
  sty PPUADDR
  sty PPUADDR
  lda #28
  sta 8
:
  jsr copyLineImg
  dec 8
  bne :-
.endif

;;; Clear nametable $2000
  lda #$20
  sta PPUADDR
  lda #0
  sta PPUADDR
  jsr clearMap

;;; Write msg
  lda #$00
  sta PPUADDR
  lda #$00
  sta PPUADDR
  sta curLine

lineLoop:
  jsr clearLineImg
  lda curLine
  cmp #28
  bcs lineLoopDone
  inc curLine
  asl a
  tax
  lda linetable,x
  tay
  lda linetable+1,x
  beq @noString
  ldx #8
  jsr vwfPuts
@noString:
  jsr copyLineImg
  jmp lineLoop
lineLoopDone:

  lda #PPUCTRL_NMI
  sta PPUCTRL

  jsr setSpriteZero
  lda #0
  sta SPRADDR
  lda #>lineImgBuf
  sta SPRDMA
  lda #0
  sta vblanked

main_loop:
  lda vblanked
  beq main_loop
  lda PPUSTATUS
  lda #0
  sta vblanked

;;; turn on display
  lda #0
  sta PPUSCROLL
  lda #$f8
  sta PPUSCROLL
  lda #PPUCTRL_NMI
  sta PPUCTRL
  lda #PPUMASK_BG0|PPUMASK_SPR
  sta PPUMASK


;;; wait for sprite 0 clear
:
  bit PPUSTATUS
  bvs :-
;;; wait for sprite 0 set
:
  bit PPUSTATUS
  bmi main_loop
  bvc :-
  lda #PPUCTRL_NMI|PPUCTRL_BGHIPAT
  sta PPUCTRL
;  lda #2
;  sta PPUSCROLL
;  lda #0
;  sta PPUSCROLL

  jmp main_loop




;;; draw_text:
;   Draws a nul-terminated line of text to the screen.
;   0: pointer to text src
;   X: high word of dst
;   Y: low word of dst
;   out: Y: length of string; A: 0
draw_text:
  stx PPUADDR
  sty PPUADDR
  ldy #0
  lda (0),y
  beq @skip
@loop:
  sta PPUDATA
  iny
  lda (0),y
  bne @loop
@skip:
  rts

setSpriteZero:
  lda #$f0
  ldx #0
:
  sta lineImgBuf,x
  inx
  bne :-
  lda #134
  sta lineImgBuf
  lda #$00
  sta lineImgBuf+1
  lda #%00100011
  sta lineImgBuf+2
  lda #$FC
  sta lineImgBuf+3
  rts


.segment "RODATA"
.align 256
chrData:
  .incbin "hello.chr"

linetable:
  .addr 0, copr, strike1, strike2, strike3, 0, line1, line2
  .addr line3, 0, lorem1, lorem2, lorem3, lorem4, lorem5, lorem6
  .addr lorem7, lorem8, lorem9, lorem10, lorem11, lorem12, lorem13, lorem14
  .addr lorem15, lorem16, lorem17, lorem18


copr:    .asciiz "VWF Demo 0.02  2006 Damian Yerrick"
strike1: .byt "[ !",$22,"#$%&'()*+,-./0123456789:;<=>?]",0
strike2: .asciiz "[@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\]^_]"
strike3: .asciiz "[`abcdefghijklmnopqrstuvwxyz{|}~]"

line1:  .asciiz "Variable width fonts are easier to read than fixed width"
line2:  .asciiz "fonts. And now that variable width text can fill the"
line3:  .asciiz "whole screen, I present Lorem Ipsum:"

lorem1:  .asciiz "Sed ut perspiciatis unde omnis iste natus error sit"
lorem2:  .asciiz "voluptatem accusantium doloremque laudantium, totam"
lorem3:  .asciiz "rem aperiam, eaque ipsa quae ab illo inventore veritatis"
lorem4:  .asciiz "et quasi architecto beatae vitae dicta sunt explicabo."
lorem5:  .asciiz "Nemo enim ipsam voluptatem quia voluptas sit aspernatur"
lorem6:  .asciiz "aut odit aut fugit, sed quia consequuntur magni dolores"
lorem7:  .asciiz "eos qui ratione voluptatem sequi nesciunt. Neque porro"
lorem8:  .asciiz "quisquam est, qui dolorem ipsum quia dolor sit amet,"
lorem9:  .asciiz "consectetur, adipisci velit, sed quia non numquam eius"
lorem10: .asciiz "modi tempora incidunt ut labore et dolore magnam"
lorem11: .asciiz "aliquam quaerat voluptatem. Ut enim ad minima veniam,"
lorem12: .asciiz "quis nostrum exercitationem ullam corporis suscipit"
lorem13: .asciiz "laboriosam, nisi ut aliquid ex ea commodi consequatur?"
lorem14: .asciiz "Quis autem vel eum iure reprehenderit qui in ea"
lorem15: .asciiz "voluptate velit esse quam nihil molestiae consequatur,"
lorem16: .asciiz "vel illum qui dolorem eum fugiat quo voluptas nulla"
lorem17: .asciiz "pariatur?"
lorem18: .asciiz "  -- Cicero, DE FINIBUS, section 1.10.32"

pal:
  .byt $30,$0f,$30,$0f,$30,$30,$0f,$0f,$30,$34,$34,$34,$30,$34,$34,$34
  .byt $30,$0f,$30,$30,$30,$30,$30,$30,$30,$30,$30,$30,$30,$30,$30,$30

@die:
  jmp @die


.segment "VECTORS"

  .addr nmihandler, main, irqhandler
