.include "nes.h"
.export clearLineImg, putTile, copyLineImg, clearMap, lineImgBuf, vwfPuts
.import chrData

.segment "BSS"
.align 256
lineImgBuf: .res 256

.segment "CODE"


;
; Clears the line image buffer.
;
clearLineImg:
  ldx #$3b
  lda #0
:
  sta lineImgBuf+$08,x
  sta lineImgBuf+$44,x
  sta lineImgBuf+$80,x
  sta lineImgBuf+$bc,x
  dex
  bpl :-

  ; Make opaque columns for sprite 0
  lda #$F0
  .repeat 8,row
    sta lineImgBuf+row
  .endrep
  lda #$0F
  .repeat 8,row
    sta lineImgBuf+$F8+row
  .endrep

  rts


;
; Puts a tile to position X.
; In:   A = tile number
;       X = destination X position
; Trash: 6-7, 11-15
.align 256
putTile:
@curHiBits = 11
@dstAddr = 12
@nShifts = 13
@tileAddr = 14
@shiftJmp = 6
  ; Find source address (33)
  sta @tileAddr
  ldy #0
  tya
  asl @tileAddr
  rol a
  asl @tileAddr
  rol a
  asl @tileAddr
  rol a
  adc #>chrData
  sta @tileAddr+1

  ; Find destination address (18)
  txa
  and #%11111000
  sta @dstAddr
  txa
  ldx @dstAddr
  and #%00000111
  cmp #5
  bcs @reverseShift
  eor #7
  sta @nShifts
  asl a
  adc @nShifts
  adc #<@shiftDuff
  sta @shiftJmp
  lda #>@shiftDuff
  sta @shiftJmp+1

@loop:
  ; Get data byte (10*8)
  lda #0
  sta @curHiBits
  lda (@tileAddr),y
  jmp (@shiftJmp)
@shiftDuff:
  lsr a
  ror @curHiBits
  lsr a
  ror @curHiBits
  lsr a
  ror @curHiBits
  lsr a
  ror @curHiBits
  lsr a
  ror @curHiBits
  lsr a
  ror @curHiBits
  lsr a
  ror @curHiBits

  ; OR byte into destination (36*8)
  ora lineImgBuf,x
  sta lineImgBuf,x
  lda @curHiBits
  ora lineImgBuf+8,x
  sta lineImgBuf+8,x
  inx
  iny
  cpy #8
  bcc @loop
  rts

@reverseShift:
  sta @nShifts
  asl a
  adc @nShifts
  adc #<@revDuff
  sta @shiftJmp
  lda #>@revDuff
  sta @shiftJmp+1

@reverseLoop:
  ; Get data byte (10*8)
  lda (@tileAddr),y
  sta @curHiBits
  lda #0
  jmp (@shiftJmp)
@revDuff:
  asl @curHiBits
  rol a
  asl @curHiBits
  rol a
  asl @curHiBits
  rol a
  asl @curHiBits
  rol a
  asl @curHiBits
  rol a
  asl @curHiBits
  rol a
  asl @curHiBits
  rol a
  asl @curHiBits
  rol a

  ; OR byte into destination
  ora lineImgBuf,x
  sta lineImgBuf,x
  lda @curHiBits
  ora lineImgBuf+8,x
  sta lineImgBuf+8,x
  inx
  iny
  cpy #8
  bcc @reverseLoop
  rts


;
; Copies the line to the screen
; in:  PPUADDR = destination address
;
copyLineImg:
  ldx #PPUMASK_RED
  stx PPUMASK
  .repeat 16,chr
    .repeat 2,bank
      .repeat 8,scanline
        lda lineImgBuf + scanline + bank*128 + chr*8
        sta PPUDATA
      .endrep
    .endrep
  .endrep
  ldx #0
  stx PPUMASK
  rts


;
; Clears a nametable for use with VWF.
; in:  PPUADDR = destination address
;
clearMap:
@nLines = 14
@base = 15

  lda #30
  sta @nLines
  lda #0
  sta @base
@loop1:
  ldx @base
  ldy #16
:
  stx PPUDATA
  inx
  dey
  bne :-
  ldx @base
  ldy #16
:
  stx PPUDATA
  inx
  dey
  bne :-
  lda @base
  clc
  adc #16
  sta @base
  dec @nLines
  bne @loop1

  ; Set up attributes
  ldy #8
  lda #0
  ldx #$55
:
  sta PPUDATA
  sta PPUDATA
  sta PPUDATA
  sta PPUDATA
  stx PPUDATA
  stx PPUDATA
  stx PPUDATA
  stx PPUDATA
  dey
  bne :-
  rts


;
; Puts a tile to position X.
; In:   AAYY = string base address
;       X = destination X position
; Trash: 6-15
;
vwfPuts:
@str = 8
@horz = 10
  sty @str
  sta @str+1
  stx @horz
@loop:
  ldy #0
  lda (@str),y
  beq @done
  ldx #PPUMASK_GREEN
  stx PPUMASK
  ldx @horz
  jsr putTile
  ldx #0
  stx PPUMASK
  ldy #0
  lda (@str),y
  tax
  lda chrWidths,x
  clc
  adc @horz
  sta @horz
  cmp #248
  bcs @done
  inc @str
  bne @loop
  inc @str+1
  bne @loop

@done:
  rts

.segment "RODATA"
.align 256
chrWidths:
  .incbin "widths.bin"
